// src/Editor.tsx
import { EditorState, StateEffect } from "@codemirror/state";
import { EditorView as EditorView2 } from "@codemirror/view";
import { jsonSchema, updateSchema } from "codemirror-json-schema";
import React, { useEffect, useMemo, useRef, useState } from "react";

// src/extensions.ts
import { autocompletion, closeBrackets, closeBracketsKeymap, completionKeymap } from "@codemirror/autocomplete";
import { defaultKeymap, history, historyKeymap, insertTab } from "@codemirror/commands";
import { bracketMatching, foldGutter, foldKeymap, indentOnInput, indentUnit } from "@codemirror/language";
import { lintGutter, lintKeymap } from "@codemirror/lint";
import { EditorView, drawSelection, gutter, highlightActiveLineGutter, keymap, lineNumbers } from "@codemirror/view";
import { vscodeDark } from "@uiw/codemirror-theme-vscode";
var commonExtensions = [
  gutter({ class: "CodeMirror-lint-markers" }),
  bracketMatching(),
  highlightActiveLineGutter(),
  // basicSetup,
  closeBrackets(),
  history(),
  autocompletion(),
  lineNumbers(),
  lintGutter(),
  indentUnit.of("	"),
  indentOnInput(),
  drawSelection(),
  foldGutter(),
  keymap.of([
    ...closeBracketsKeymap,
    ...defaultKeymap,
    ...historyKeymap,
    ...foldKeymap,
    ...completionKeymap,
    ...lintKeymap,
    { key: "Tab", run: insertTab },
    { key: "Shift-Tab", run: insertTab }
  ]),
  // style
  vscodeDark,
  EditorView.lineWrapping
];

// src/Editor.tsx
function cleanedMapStr(mapString) {
  if (!mapString) {
    return "";
  }
  return JSON.stringify(JSON.parse(mapString));
}
function getRandomInt(max) {
  return Math.floor(Math.random() * max);
}
function Editor(props) {
  const { value, onChange, schema, height = 300, tabSize = 4, extensions } = props;
  const editorRef = useRef(null);
  const [domId] = useState(`json-schema-editor-${getRandomInt(1e3)}`);
  const customExtensions = useMemo(() => {
    return [
      EditorState.tabSize.of(tabSize),
      EditorView2.theme({
        "&": {
          height
        }
      })
    ];
  }, [tabSize, height]);
  useEffect(() => {
    const updateListener = EditorView2.updateListener.of((update) => {
      if (update.docChanged) {
        onChange(update.state.doc.toString());
      }
    });
    const lastExtensions = [
      commonExtensions,
      customExtensions,
      jsonSchema(schema),
      updateListener,
      extensions || []
    ];
    if (!editorRef.current) {
      const state = EditorState.create({
        doc: value,
        extensions: lastExtensions
      });
      editorRef.current = new EditorView2({
        state,
        parent: document.querySelector(`#${domId}`)
      });
    } else {
      editorRef.current.dispatch({
        effects: StateEffect.reconfigure.of(lastExtensions)
      });
    }
  }, [commonExtensions, customExtensions, extensions]);
  useEffect(() => {
    if (cleanedMapStr(value) !== cleanedMapStr(editorRef.current?.state.doc.toString())) {
      editorRef.current?.dispatch({
        changes: { from: 0, to: editorRef.current?.state.doc.length, insert: value }
      });
    }
  }, [value]);
  useEffect(() => {
    if (editorRef.current && schema) {
      updateSchema(editorRef.current, schema);
    }
  }, [schema]);
  return /* @__PURE__ */ React.createElement("div", { id: domId });
}
export {
  Editor,
  commonExtensions
};
